package org.expressolivre.cte.pages.email;

import org.apache.commons.lang.StringUtils;
import org.expressolivre.cte.pages.common.Page;
import org.openqa.selenium.Alert;
import org.openqa.selenium.By;
import org.openqa.selenium.JavascriptExecutor;
import org.openqa.selenium.RenderedWebElement;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.FindBy;

/**
 * @author L.F.Estivalet (Serpro)
 * 
 *         Created on Dec 16, 2010 at 9:43:21 AM
 * 
 */
public class MailPage extends Page {

	@FindBy(xpath = "//table[@id='folders_tbl']/tbody/tr[1]/td/table/tbody/tr[2]/td/div/span")
	private WebElement newEmail;

	@FindBy(id = "to_1")
	private WebElement to;

	@FindBy(id = "subject_1")
	private WebElement subject;

	@FindBy(id = "send_button_1")
	private WebElement send;

	@FindBy(id = "important_message_1")
	/** Flag de importante no momento de compor um email. */
	private WebElement important;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[2]/span[3]")
	/** Flag de importante de um email aberto. */
	private WebElement importantFlag;

	@FindBy(id = "span_flag_FLAGGED")
	/** Filtra emails importantes. */
	private WebElement importantFilter;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[2]/span[3]")
	private WebElement normalFlag;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[2]/span[2]")
	private WebElement notReadFlag;

	@FindBy(id = "return_receipt_1")
	private WebElement returnReceipt;

	@FindBy(id = "a_cco_link_1")
	private WebElement blindCarbonCopyLink;

	@FindBy(id = "cco_1")
	private WebElement blindCarbonCopy;

	@FindBy(id = "a_cc_link_1")
	private WebElement carbonCopyLink;

	@FindBy(id = "cc_1")
	private WebElement carbonCopy;

	@FindBy(id = "save_message_options_1")
	private WebElement saveEmail;

	@FindBy(linkText = "Anexos: adicionar+")
	private WebElement attachmentLink;

	@FindBy(id = "inputFile_1_1")
	private WebElement attachment;

	@FindBy(id = "inputFile_1_2")
	private WebElement attachment2;

	@FindBy(xpath = "//div[@id='footer_menu']/table[@id='footer_box']/tbody/tr[@id='table_message']/td[@id='span_options']/span[1]/span")
	private WebElement deleteEmailLink;

	@FindBy(id = "lINBOX/Trashtree_folders")
	private WebElement trashFolder;

	@FindBy(id = "empty_trash")
	private WebElement emptyTrashLink;

	@FindBy(id = "divScrollMain_0")
	private WebElement emailsListDiv;

	@FindBy(id = "tot_m")
	private WebElement totalEmails;

	@FindBy(xpath = "//div[@id='exmail_main_body']/table[1]/tbody[@id='border_tbody']/tr[1]/td[2]/table/tbody/tr/td[1]")
	private WebElement emailTab;

	@FindBy(id = "chk_box_select_all_messages")
	private WebElement selectAllEmails;

	@FindBy(xpath = "//td[@id='link_tools']/div/span")
	private WebElement toolsMenu;

	@FindBy(xpath = "//div[@id='menu_tools']/table/tbody/tr[4]/td")
	private WebElement editFolder;

	@FindBy(xpath = "//input[@value='Nova Pasta']")
	private WebElement createNewFolder;

	@FindBy(xpath = "//input[@value='Excluir pasta']")
	private WebElement deleteFolder;

	@FindBy(xpath = "//span[@onclick='wfolders.makeWindow(\"\",\"move_to\")']")
	/** Opcao de mover do rodape. */
	private WebElement moveEmailLink;

	@FindBy(id = "dJSWin_wfolders_bok")
	private WebElement moveEmailButton;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[3]/span[3]")
	private WebElement forwardEmail;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[3]/span[5]")
	private WebElement replyEmail;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[2]/td/div/span[1]")
	private WebElement replyAllEmail;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[2]/td/div/span[2]")
	private WebElement replyEmailWithoutHistory;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[2]/td/div/span[3]")
	private WebElement replyAllEmailWithoutHistory;

	@FindBy(xpath = "//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr/td[2]/span[2]/span[1]")
	private WebElement editEmail;

	/**
	 * @param driver
	 */
	public MailPage(WebDriver driver) {
		super(driver);
	}

	public void createNewEmail() {
		this.newEmail.click();
		waitForElement(By.id("to_1"));
	}

	public void setSubject(String subject) {
		this.subject.sendKeys(subject != null ? subject : "");
	}

	public void setBody(String body) {
		// Foi o unico jeito que consegui pegar o richtexteditor...
		if (body != null) {
			((JavascriptExecutor) driver)
					.executeScript("document.getElementById('body_1').contentWindow.document.body.innerHTML='"
							+ body + "';");
		}
	}

	/**
	 * @param to
	 * @param subject
	 * @param body
	 */
	public void composeEmail(String to, String subject, String body) {
		this.createNewEmail();
		this.to.sendKeys(to);
		this.setSubject(subject);
		this.setBody(body);
	}

	public String readEmail(String subject) {
		// waitForElementByXpath("//div[@id='exmail_main_body']/table[1]/tbody[@id='border_tbody']/tr[1]/td[2]/table/tbody/tr/td[1]");
		String id = findEmailIdBySuject(subject);
		id = id.substring(id.lastIndexOf("_") + 1);
		clickElement(By.id("td_who_" + id));
		super.waitForElement(By.id("div_message_scroll_" + id + "_r"));
		return id;
	}

	public void editEmail() {
		super.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr/td[2]/span[2]/span[1]"));
		// div[@id='exmail_main_body']/div[@id='content_id_9_r']/table/tbody/tr[1]/td/table/tbody/tr/td[2]/span[1]
		this.editEmail.click();
	}

	public void editEmail(String id) {
		super.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr/td[2]/span[2]/span[1]"));
		// div[@id='exmail_main_body']/div[@id='content_id_9_r']/table/tbody/tr[1]/td/table/tbody/tr/td[2]/span[1]
		this.editEmail.click();
		waitForElement(By.id("content_id_" + id));
	}

	/**
	 * Encaminha uma mensagem.
	 * 
	 * @param id
	 *            Id da mensagem a ser encaminhada
	 * @param to
	 *            Destinatario
	 */
	public void forwardEmail(String id, String to) {
		super.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[3]/span[3]"));
		this.forwardEmail.click();
		super.waitForElement(By.id("to_" + id));
		driver.findElement(By.id("to_" + id)).sendKeys(to);
		driver.findElement(By.id("send_button_" + id)).click();
	}

	private void reply(String id) {
		super.waitForElement(By.id("to_" + id));
		driver.findElement(By.id("send_button_" + id)).click();
	}

	/**
	 * Responde uma mensagem.
	 * 
	 * @param id
	 *            Id da mensagem a ser respondida
	 */
	public void replyEmail(String id) {
		super.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[3]/span[5]"));
		this.replyEmail.click();
		this.reply(id);
	}

	/**
	 * Responde uma mensagem.
	 * 
	 * @param id
	 *            Id da mensagem a ser respondida
	 */
	public void replyAllEmailWithoutHistory(String id) {
		super.findAndClickElement("msg_opt_reply_options_" + id + "_r");
		super.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[2]/td/div/span[3]"));
		this.replyAllEmailWithoutHistory.click();
		this.reply(id);
	}

	/**
	 * Responde uma mensagem.
	 * 
	 * @param id
	 *            Id da mensagem a ser respondida
	 */
	public void replyEmailWithoutHistory(String id) {
		super.findAndClickElement("msg_opt_reply_options_" + id + "_r");
		super.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[2]/td/div/span[2]"));
		this.replyEmailWithoutHistory.click();
		this.reply(id);
	}

	/**
	 * Responde uma mensagem.
	 * 
	 * @param id
	 *            Id da mensagem a ser respondida
	 */
	public void replyAllEmail(String id) {
		super.findAndClickElement("msg_opt_reply_options_" + id + "_r");
		super.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[2]/td/div/span[1]"));
		this.replyAllEmail.click();
		this.reply(id);
	}

	/**
	 * Abre uma pasta de email especifica.
	 * 
	 * @param folder
	 *            Pasta de email a ser aberta.
	 */
	public void openFolder(String folder) {
		clickElement(By.id(folder));
		waitForElement(By.id("divScrollMain_0"));
		waitForElement(By
				.xpath("//div[@id='divScrollMain_0']/table[@id='table_box']/tbody[@id='tbody_box']/tr[1]/td[1]/input[1]"));
	}

	/**
	 * Abre uma pasta de email especifica.
	 * 
	 * @param folder
	 *            Pasta de email a ser aberta.
	 */
	public void openFolder(WebElement folder) {
		folder.click();
		waitForElement(By.id("divScrollMain_0"));
	}

	public void setImportantFilter() {
		this.importantFilter.click();
	}

	/**
	 * Limpa a lixeira.
	 */
	public void cleanupTrash() {
		this.openFolder(this.trashFolder);
		this.emptyTrashLink.click();
	}

	/**
	 * @return Numero de emails na pasta selecionada.
	 */
	public int getTotalEmails() {
		return Integer.valueOf(this.totalEmails.getText());
	}

	public void moveEmail(String folderName) {
		this.moveEmailLink.click();
		super.findAndClickElement("lINBOX/" + folderName + "wfolders_tree");
		this.moveEmailButton.click();
	}

	public void editFolder() {
		this.toolsMenu.click();
		this.waitForElement(By
				.xpath("//div[@id='menu_tools']/table/tbody/tr[4]/td"));
		this.editFolder.click();
	}

	public void createNewFolder(String folderName) {
		this.editFolder();
		this.createNewFolder.click();
		Alert newFolder = driver.switchTo().alert();
		assert "Insira o nome da nova pasta: ".equals(newFolder.getText());
		newFolder.sendKeys(folderName);
		newFolder.accept();
	}

	public void deleteFolder(String folderName) {
		this.editFolder();
		this.driver.findElement(By.id("lINBOX/" + folderName + "folders_tree"))
				.click();
		this.deleteFolder.click();
		Alert confirm = driver.switchTo().alert();
		assert ("Você gostaria de excluir a pasta: " + folderName + "?")
				.equals(confirm.getText());
		confirm.accept();
	}

	/**
	 * Marca email sendo composto como importante.
	 */
	public void checkImportantFlag() {
		this.important.click();
	}

	public void setImportantFlag() {
		this.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[2]/span[3]"));
		this.importantFlag.click();
	}

	public void setNormalFlag() {
		this.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[2]/span[3]"));
		this.normalFlag.click();
	}

	public void setNotReadFlag() {
		this.waitForElement(By
				.xpath("//div[@id='exmail_main_body']/div[2]/table/tbody/tr[1]/td/table/tbody/tr[1]/td[2]/span[2]"));
		this.notReadFlag.click();
	}

	/**
	 * Marca solicitacao de confirmacao de leitura do email sendo composto.
	 */
	public void checkReturnReceipt() {
		this.returnReceipt.click();
	}

	/**
	 * Salva email sendo composto (devera ir para a pasta de Rascunhos).
	 * 
	 * TODO adicionar teste para verificar se o email realmente esta presente na
	 * pasta Rascunhos??
	 */
	public void saveEmail() {
		this.saveEmail.click();
	}

	/**
	 * Apaga email previamente selecionado.
	 */
	public void deleteEmail() {
		this.deleteEmailLink.click();
	}

	public void selectAllEmails() {
		this.selectAllEmails.click();
	}

	/**
	 * Adiciona email no campo de copia oculta.
	 * 
	 * @param email
	 *            Email a ser adicionado.
	 */
	public void setBlindCarbonCopy(String email) {
		this.blindCarbonCopyLink.click();
		this.blindCarbonCopy.sendKeys(email);
	}

	/**
	 * Adiciona email no campo de copia.
	 * 
	 * @param email
	 *            Email a ser adicionado.
	 */
	public void setCarbonCopy(String email) {
		this.carbonCopyLink.click();
		this.carbonCopy.sendKeys(email);
	}

	/**
	 * Anexo um arquivo no email sendo composto.
	 * 
	 * @param attachment
	 *            Caminho para o arquivo a ser anexado.
	 */
	public void addAttachment(String attachment) {
		this.attachmentLink.click();
		this.attachment.sendKeys(attachment);
	}

	public void addAttachment2(String attachment) {
		this.attachmentLink.click();
		this.attachment2.sendKeys(attachment);
	}

	/**
	 * Envia email sendo composto.
	 */
	public void sendMail() {
		this.send.click();
	}

	/**
	 * Fecha aba do email sendo composto e confirma que email nao esta salvo.
	 */
	public void closeMail() {
		this.closeMail(true);
	}

	/**
	 * @param checkAlert
	 */
	public void closeMail(boolean checkAlert) {

		// Na versao Serpro:
		// td[@id='border_id_1']/table/tbody/tr/td[2]/img

		// Na versao Comunidade:
		// td[@id='border_id_1']/div/div[2]/img

		// Resolvi usando o xpath abaixo:
		super.clickElement(By.xpath("//td[@id='border_id_1']//img"));

		if (checkAlert) {
			Alert closeAlert = driver.switchTo().alert();
			closeAlert.accept();
		}
	}

	public void sendMail(String id) {
		super.findAndClickElement("send_button_" + id);
	}

	/**
	 * @param to
	 * @param subject
	 */
	public void composeEmail(String to, String subject) {
		composeEmail(to, subject, null);
	}

	/**
	 * @param to
	 */
	public void composeEmail(String to) {
		composeEmail(to, null, null);
	}

	/**
	 * Procura um email pelo assunto.
	 * 
	 * @param subject
	 *            Assunto a ser procurado.
	 * @param select
	 *            Se email encontrado, deseja seleciona-lo?
	 * @return "id" do email.
	 */
	public String findEmailIdBySuject(String subject, boolean select) {
		String html = driver.getPageSource();
		String part1 = StringUtils.substringBefore(html, subject);
		int start = StringUtils.lastIndexOf(part1, "id=\"") + 4;
		int last = StringUtils.indexOf(part1, "\"", start);
		String id = StringUtils.substring(part1, start, last);

		if (select) {
			// Marca checkbox correspondente ao id da mensagem.
			clickElement(By.id("check_box" + id.substring(1)));
		}
		return id;
	}

	/**
	 * @return Ids de todos os emails da pasta selecionada.
	 */
	public String[] getEmailIds() {
		String[] ids = new String[this.getTotalEmails()];
		for (int i = 1; i <= this.getTotalEmails(); i++) {
			String id = driver
					.findElement(
							By.xpath("//div[@id='divScrollMain_0']/table[@id='table_box']/tbody[@id='tbody_box']/tr["
									+ i + "]/td[1]/input[1]")).getAttribute(
							"id");
			ids[i - 1] = id.substring(id.lastIndexOf("_") + 1);
		}
		return ids;
	}

	/**
	 * @param emailId
	 *            Id do email.
	 * @return <code>true</code> se email importante, <code>false</code> caso
	 *         contrario.
	 */
	public boolean isImportant(String emailId) {
		return super
				.isDisplayed((RenderedWebElement) driver.findElement(By
						.xpath("//div[@id='exmail_main_body']/div[@id='content_id_0']/div[@id='divScrollMain_0']/table[@id='table_box']/tbody[@id='tbody_box']/tr[@id='"
								+ emailId + "']/td[5]/img")));
	}

	public String findEmailIdBySuject(String subject) {
		return findEmailIdBySuject(subject, false);
	}

	/**
	 * Verifica se mensagem informada corresponde ao exibido pelo Expresso na
	 * div onde sao exibidas mensagens ao usuario.
	 * 
	 * @param message
	 *            Mensagem a ser validada.
	 */
	public void assertMessage(String message) {
		assert message.equals(getDisplayedElementContent("em_div_write_msg"));
	}

	/**
	 * @return the to
	 */
	public WebElement getTo(String id) {
		waitForElement(By.id("to_" + id));
		// findAndClickElement("to_" + id);
		// return getDisplayedElementContent(id);
		return driver.findElement(By.id("to_" + id));
	}

	/**
	 * @return the subject
	 */
	public WebElement getSubject(String id) {
		waitForElement(By.id("subject_" + id));
		return driver.findElement(By.id("subject_" + id));
	}

	/**
	 * @return the blindCarbonCopy
	 */
	public WebElement getBlindCarbonCopy(String id) {
		waitForElement(By.id("cco_" + id));
		return driver.findElement(By.id("cco_" + id));
	}

	/**
	 * @return the carbonCopy
	 */
	public WebElement getCarbonCopy(String id) {
		waitForElement(By.id("cc_" + id));
		return driver.findElement(By.id("cc_" + id));
	}

	public Object getBody(String id) {
		waitForElement(By.id("body_" + id));
		return ((JavascriptExecutor) driver)
				.executeScript("return document.getElementById('body_" + id
						+ "').contentWindow.document.body.innerHTML;");
	}

}
